unit hiIconAnima; {       ver 1.00 }

interface

uses Windows,Share,Debug,Kol;

type
  ThiIconAnima = class(TDebug)
   private
    FListIndex: PStrListEx;
    FListBitmap: PImageList;
    FIcon: PIcon;
    FBitmap: PBitmap; 
    FTimer: PTimer;
    FDelay: integer;
    FImgSize: Integer;

    procedure _OnTimer(Obj:PObj);    
    procedure SetIndex(Value:PStrListEx);
    procedure SetBitmap(Value:PStrListEx);    
    procedure SetImgSize(Value:integer);
   public

    _event_onIconFrame: THI_event;
    _event_onBitmapFrame: THI_event;

    _data_Index: THI_event;

    _prop_MaskColor : integer;
    _prop_TranspColor : integer;
    _prop_CircleAnima : boolean;
    _prop_Index       : integer;

    property _prop_IndexArray : PStrListEx write SetIndex;    
    property _prop_Bitmaps    : PStrListEx write SetBitmap;
    property _prop_ImgSize    : integer write SetImgSize;
    property _prop_Delay      : integer read FDelay write FDelay;

    constructor Create;
    destructor Destroy; override;

    procedure _work_doAnima(var _Data:TData; Index:word);
    procedure _work_doFrame(var _Data:TData; Index:word);
    procedure _work_doStop(var _Data:TData; Index:word);
    procedure _work_doDelay(var _Data:TData; Index:word);

    procedure _var_CurFrame(var _Data:TData; Index:word);
    procedure _var_FrameCount(var _Data:TData; Index:word);
    procedure _var_IndexCount(var _Data:TData; Index:word);    

  end;

var
   FStop     : boolean = false;
   FCurFrame : integer = 0;
   FCurIndex : integer = 0;
implementation

constructor ThiIconAnima.Create;
begin
   inherited;
   FIcon := NewIcon;
   FBitmap := NewBitmap(0,0);
   FTimer := NewTimer(83);
   FTimer.Enabled := false;
   FTimer.OnTimer := _onTimer;
end;

destructor ThiIconAnima.Destroy;
begin
   FTimer.free;
   FIcon.free;
   FBitmap.free;
   if Assigned(FListIndex) then FListIndex.free; 
   if Assigned(FListBitmap) then FListBitmap.free;
   inherited;
end;

procedure ThiIconAnima._OnTimer;
var      db,di:TData;
begin
   if FStop then begin
      FTimer.Enabled := false;
      exit;
   end;

   if not (Assigned(FListIndex) and Assigned(FListBitmap)) then exit;

   FIcon.Clear;
   FBitmap.Clear;
      
   if (FCurIndex >= 0) and (FCurIndex < FListIndex.Count) then begin   
      FCurFrame := Integer(FListIndex.Objects[FCurIndex]);
      if (FCurFrame >= 0) and (FCurFrame < FListBitmap.Count) then
         FIcon.Handle := FListBitmap.ExtractIcon(FCurFrame);
   end;

   if FIcon.Handle = 0 then exit;
    
   FBitmap.Handle := FIcon.Convert2Bitmap(_prop_TranspColor);

   dtBitmap(db,FBitmap);
   dtIcon(di,FIcon);
   inc(FCurIndex);
   if FCurIndex > FListIndex.Count - 1 then begin
      if _prop_CircleAnima then
         FCurIndex := 0
      else begin
         FStop := true;
         FCurIndex := FListIndex.Count - 1; 
      end;   
   end;
   _hi_OnEvent(_event_onIconFrame, di);
   _hi_OnEvent(_event_onBitmapFrame, db);
end;

procedure ThiIconAnima.SetIndex;
begin
   FListIndex := NewStrListEx;
   FListIndex := Value;
end;

procedure ThiIconAnima.SetImgSize;
begin
   FImgsize := Value;
   if FImgsize < 16 then FImgsize:= GetSystemMetrics(SM_CXICON);
end;

procedure ThiIconAnima.SetBitmap;
var   Icon: PIcon;
      IconInfo: TIconInfo;
      bmp,mask,body: PBitmap;
      i: integer;      
begin
   FListBitmap := NewImageList(nil);
   for i := 0 to Value.Count - 1 do begin 
      FListBitmap.ImgWidth := FImgSize;
      FListBitmap.ImgHeight := FImgSize;
      FListBitmap.DrawingStyle := [dsTransparent];
      
      bmp := NewBitmap(FImgSize,FImgSize);
      bmp.Handle := Value.Objects[i];
      body := NewBitmap(0,0);
      body.Assign(bmp);
      mask := NewBitmap(FImgSize,FImgSize);
      mask.Assign(bmp);
      mask.Convert2Mask(Color2RGB(_prop_MaskColor));

      IconInfo.fIcon := true;
      IconInfo.xHotspot := 0;
      IconInfo.yHotspot := 0;
      IconInfo.hbmMask := mask.Handle;
      IconInfo.hbmColor := body.Handle;
      Icon := NewIcon;
      Icon.Handle := CreateIconIndirect(IconInfo);
      FListBitmap.AddIcon(Icon.Handle);
      mask.Free;
      body.Free;   
      bmp.free;
      Icon.free;
   end;
end;

procedure ThiIconAnima._work_doAnima;
begin
   FTimer.Interval := FDelay;
   FCurIndex := ReadInteger(_Data, _data_Index, _prop_Index);
   FStop := false;
   FTimer.Enabled := true;
end;

procedure ThiIconAnima._work_doFrame;
begin
   FCurIndex := ReadInteger(_Data, _data_Index, _prop_Index);
   FStop := false;
   _OnTimer(nil);
end;

procedure ThiIconAnima._work_doStop; begin FStop := true; end;
procedure ThiIconAnima._work_doDelay; begin FDelay := ToInteger(_Data); end;
procedure ThiIconAnima._var_CurFrame; begin dtInteger(_Data, FCurFrame); end;

procedure ThiIconAnima._var_FrameCount;
begin
   if not Assigned(FListBitmap) then exit;
   dtInteger(_Data, FListBitmap.Count);
end;

procedure ThiIconAnima._var_IndexCount;
begin
   if not Assigned(FListIndex) then exit;
   dtInteger(_Data, FListIndex.Count);
end;

end.